import six


class Region(object):
    """
        A representation of a region. When using this, the ``id`` is looked up
        in CMP and associated with a name and optional geographical data.

        Args:
            id (str): id
    """

    def __init__(self, id=None):
        self._id = id
        if self._id is None or not isinstance(self._id, six.string_types):
            raise ValueError('id must be a string and have a value')

    def serialize(self):
        """Serialize the contents"""

        return {'id': self._id}


class Locations(object):
    """
        A representation of a list of locations. Flexible in terms of what parts
        can be set based on the spec defined for locations.

        Args:
            locations (list): list of location dicts with keys as specified below:
            location (dict): keys id, name, latitude, longitude - optional
            city (dict): keys id, name, latitude, longitude - optional
            state (dict): keys id, name, latitude, longitude - optional
            country (dict): keys id, name, latitude, longitude - optional
            region (dict): keys id, name, latitude, longitude - optional
            type (str): one of 'customer' or 'provider' - optional

        Requirement for each lower level dict is:
            id (str)
            name (str) - one of id or name is required
            latitude (float) - optional
            longitude (float) - optional

        Example:
            This shows how to associate a paris location::

                locations = Locations([{
                    'country': {'name': 'France'},
                    'city': {'name': 'Paris'},
                    'location': {
                        'name': 'Legendary Paris Place',
                        'latitude': 48.860764,
                        'longitude': 2.393646,
                    }
                }])

                server = Server(
                    id='server-2',
                    name='Server with lots of details',
                    locations=locations
                )

    """  # noqa

    def __init__(self, locations=None):
        if locations is None:
            raise ValueError('locations must be a list of location hashes')
        self._data = []
        for location in locations:
            for attr in ('location', 'city', 'state', 'country', 'region'):
                if attr in location:
                    elem = location[attr]
                    if 'id' not in elem and 'name' not in elem:
                        raise ValueError('one of id or name must be specified')
                    if 'id' in elem and not isinstance(elem['id'],
                                                       six.string_types):
                        raise ValueError('id must be a string')
                    if 'name' in elem and not isinstance(elem['name'],
                                                         six.string_types):
                        raise ValueError('name must be a string')

            self._data.append(location)

    def serialize(self):
        """Serialize the contents"""

        return self._data



class IpAddress(object):
    """
        A representation of an IP address

        Args:
            ip_address (str): IPv4 or IPv6 address
            description (str): Description of the ip address. Useful when there is no network, e.g. if the IP address is public.
            network_id (str): Optional network id. This isn't used yet, but may be used in the future to associate an IP address with a network.
            network_name (str): Optional network name. This isn't used yet.

    """  # noqa

    def __init__(self, ip_address=None, network_id=None, network_name=None,
                 description=None):
        self._ip_address = ip_address
        self._network_id = network_id
        self._network_name = network_name
        self._description = description

    def serialize(self):
        """Serialize the contents"""

        results = {}
        if self._ip_address is not None:
            results['ip_address'] = self._ip_address

        if self._network_id is not None and self._network_name is not None:
            results['network'] = {
                'id': self._network_id,
                'name': self._network_name,
            }

        if self._description is not None:
            results['description'] = self._description

        return results



from rfc3339 import convert_datetime  # noqa
from tasks import set_task_percentage  # noqa
from metadata import Metadata  # noqa
from connections import Connections  # noqa
from image_detail import ImageDetail, ImageDetailMap  # noqa
from ip_address import IpAddress  # noqa
from locations import Region, Locations  # noqa
from resource import Resource  # noqa
from appliance import Appliance  # noqa
from network import Network  # noqa
from server import Server  # noqa
from service_offering import ServiceOffering  # noqa
from compute_pool import ComputePool  # noqa
from volume import Volume  # noqa
from account import Account  # noqa
from saas_user import SaasUser  # noqa
from tools import serialize_list, vcr_cassette_context  # noqa
from colo_space import ColoSpace  # noqa

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



class Connections(object):
    """
        A representation of inter-resource associations.

        Args:
            type (str): The type of resource to add connections to, e.g. ``server``
            connections (list): When used together with ``type``, a list of associated resource ids
                                or dicts
            appliances (list): an optional list of appliance ids
            servers (list): an optional list of server ids
            networks (list): an optional list of network ids
            volumes (list): an optional list of volume ids

        Examples:
            Create a bunch of connections to a server::

                Connections(type='server', connections=['server-1', 'server-2'])

            Create a connection to two volumes and a network::

                Connections(networks=['network-1'], volumes=['volume-1', 'volume-2'])

            Incrementally add connections::

                c = Connections()
                c.add(type='appliances', connections='appliance-1').add(servers=['server-1'])
                c.add(networks=['network-1'])

    """  # noqa

    def __init__(self, type=None, connections=None, appliances=None,
                 servers=None, networks=None, volumes=None):
        self._data = {}
        self.add(type=type, connections=connections, appliances=appliances,
                 servers=servers, networks=networks, volumes=volumes)

    def _add_type(self, type_, connections):
        if connections is None or len(connections) == 0:
            return

        if type_ not in self._data:
            self._data[type_] = []

        type_data = self._data[type_]
        for id_ in connections:
            if type_ == 'any':
                type_data.append(id_)
            else:
                type_data.append({'id': id_})

    def add(self, type=None, connections=None, appliances=None, servers=None,
            networks=None, volumes=None):
        """
            Add a connection

            Args:
                type (str): The type of resource to add connections to, e.g. ``server``
                connections (str): When used together with ``type``, a list of associated resource ids
                appliances (str): an optional list of appliance ids
                servers (str): an optional list of server ids
                networks (str): an optional list of network ids
                volumes (str): an optional list of volume ids

            Returns:
                :py:class:`nflex_connector_utils.connections.Connections`: returns itself, so that ``add`` methods can be chained
        """  # noqa

        if type is not None and connections is not None:
            self._add_type(type, connections)

        self._add_type('appliance', appliances)
        self._add_type('server', servers)
        self._add_type('network', networks)
        self._add_type('volume', volumes)

        return self

    def serialize(self):
        """Serialize the contents"""

        return self._data



from . import Resource


class ServiceOffering(Resource):
    """
        A representation of a service offering

        Args:
            base (base): See :py:class:`nflex_connector_utils.resource.Resource` for common resource args.
            type_id: Type of service offering. Free text.

    """  # noqa

    def __init__(self, type_id=None, **kwargs):
        super(ServiceOffering, self).__init__(type='service_offering',
                                              **kwargs)
        self._type_id = type_id

    def serialize(self):
        """Serialize the contents"""

        data = super(ServiceOffering, self).serialize()

        data['details'] = {
            self.type: {
                "type_id": self._type_id,
            }
        }

        return data
